@@ -1,4 +1,4 @@
-# $Id: FileOpt.pm,v 1.111 2011/07/01 19:59:34 pfeiffer Exp $
+# $Id: FileOpt.pm,v 1.114 2012/02/07 22:26:15 pfeiffer Exp $
 
 =head1 NAME
 
@@ -506,10 +506,9 @@ sub set_rule {
     return;
   }
 
-  my $rule_is_builtin = ($rule->source =~ /\bmakepp_builtin_rules\.mk:/);
-
-  exists $finfo->{xPHONY} &&	# If we know this is a phony target, don't
-    $rule_is_builtin and	# ever let a builtin rule attempt to build it.
+  my $rule_is_builtin = ($rule->source =~ /\bmakepp_builtin_rules\.mk:/) and
+    exists $finfo->{xPHONY} and	# If we know this is a phony target, don't
+				# ever let a builtin rule attempt to build it.
       return;
 
   if( my $oldrule = $finfo->{RULE} ) {	# Is there a previous rule?
@@ -841,11 +840,11 @@ sub build_info_fname { "$_[0]{'..'}{FULLNAME}/$build_info_subdir/$_[0]{NAME}.mk"
 sub grok_build_info_file {
   my( $fh ) = @_;
   my %build_info;
-  for( <$fh> ) {		# Read another line.  TODO: Why do some tests fail with 'while'?
+  for( <$fh> ) {		# Read another line, localizing $_ (while does not)
     return unless /(.+?)=(.*)/;	# Check the format.
     return \%build_info if $1 eq 'END';
     ($build_info{$1} = $2) =~
-      tr/\cC\r/\n/d;		# Strip out the silly windows-format lines too.
+      tr/\cC\r/\n/d;		# Strip out the silly Windows EOLs too.
   }
   undef;
 }
