@@ -1,4 +1,4 @@
-# $Id: Lexer.pm,v 1.46 2010/12/10 22:08:15 pfeiffer Exp $
+# $Id: Lexer.pm,v 1.49 2012/03/04 13:56:35 pfeiffer Exp $
 
 =head1 NAME
 
@@ -155,7 +155,7 @@ sub lex_rule {
     my $bq1 = -10;		# pretend initial replacement, length of -backquote
     my $extra = '';
     while( ($bq1 = Mpp::Text::index_ignoring_single_quotes $action, '`', $bq1 + 10) >= 0 ) {
-      my $bq2 = Mpp::Text::index_ignoring_single_quotes $action, '`', $bq1 + 1; # If not found, this is convently -1, eos
+      my $bq2 = Mpp::Text::index_ignoring_single_quotes $action, '`', $bq1 + 1; # If not found, this is conveniently -1, eos
       $extra .= ';' .substr
 	substr( $action, $bq1, $bq2 - $bq1 + 1, '-backquote' ), # Make parser's life easier
 	1, -1;			# append content of backquote, so it will get parsed as normal commands
@@ -193,8 +193,13 @@ sub lex_rule {
 	$command =~ s/\d$//;	# strip off "2" in "2> file"
 	next if $expr =~ /^\&/;	# Ignore ">&2" etc.
 	$expr =~ s/^\s*//;
-	my $file = unquote +(split_on_whitespace $expr)[0] or
-	  die "Undefined redirection in rule " . $rule->source . "\n";
+	my( $file ) = split_on_whitespace $expr;
+	unless( defined $file && length( $file = unquote $file )) {
+				# something weird like bash <(cmd) syntax?
+	  Mpp::log LEX_REDIRECT => $rule
+	    if $Mpp::log_level;
+	  next;
+	}
 	next if $file eq '/dev/null' or $file =~ m|[^-+=@%^\w:,./]|;
 	if( $is_in ) {
 	  add_optional_dependency( $dir, file_info( $dir, $rule->makefile->{CWD} ), $rule, $file );
@@ -355,8 +360,14 @@ sub find_command_parser {
 	$parser = 0 unless UNIVERSAL::isa $parser, 'Mpp::CommandParser';
 				# This is assumed to mean that calling the parser already did the scanning.
       } elsif( $parser ) {	# p_skip_word or p_shell
-	shift @rest while @rest && $rest[0] =~ /^["'\\]?-/; # skip options
+	my $_c;
+	$_c ||= $1, shift @rest while @rest && $rest[0] =~ /^["'\\]?[-+](c)?/; # skip options
 	if( $parser == &Mpp::Subs::p_shell ) {
+	  unless( $_c || $firstword !~ /sh/ ) { # Calling sh script ..., optional because we might have an option arg.
+	    add_optional_dependency( $dir, file_info( $dir, $rule->makefile->{CWD} ), $rule, $rest[0] );
+	    $$found = 1;
+	    return 0;
+	  }
 	  unshift @actions, defined $from_rule ? \1 : undef, # special marker to not propagate parser into subcommand
 	      undef, undef, join ' ', ($dir eq '.' ? () : "cd $dir;"), map unquote, @rest;
 				# just 1 arg for "sh -c cmd", but handle "sh script arg..." or "eval cmd1\; cmd2"
