@@ -1,4 +1,4 @@
-# $Id: Makefile.pm,v 1.153 2011/09/15 21:08:07 pfeiffer Exp $
+# $Id: Makefile.pm,v 1.161 2012/02/28 21:41:18 pfeiffer Exp $
 package Mpp::Makefile;
 
 use Mpp::Glob qw(wildcard_do);
@@ -70,7 +70,7 @@ C<$makefile_line>.
 
 =cut
 
-my $expand_bracket;
+our $expand_bracket;
 my $makepp_simple_concatenation_seen; # Calculate only if set to a not-false value (even though a ref may end up false)
 sub expand_text {
   -1 < index $_[1], '$' or return $_[1]; # No variables ==> no substitution,
@@ -248,7 +248,7 @@ sub expand_expression {
     my( $rtn, $rest_of_line ) = ($1, $2);
     my $orig = $rtn;
     my $code = $rtn =~ tr/-/_/ && *{"$self->{PACKAGE}::f_$rtn"}{CODE} ||
-				# Convert - into _ so it's more perl friendly.
+				# Convert - into _ so it's more Perl friendly.
       *{"$self->{PACKAGE}::f_$orig"}{CODE};
 				# See if it's a known function.
     if( $code ) {
@@ -297,16 +297,15 @@ sub expand_expression {
     open my $fh, '<', $temp or die;
     local $/;
     $result = <$fh>;
-    $result =~ s/\r?\n/ /g # Get rid of newlines.
+    $result =~ s/\r?\n/ /g	# Get rid of newlines.
       unless $s_define;
     $result =~ s/\s+$//;	# Strip out trailing whitespace.
   } elsif( $expr =~ /^([^:#=]+):([^=]+)=([^=]+)$/ ) {
-				# Substitution reference (e.g., 'x:%.o=%.c')?
-    my $from = (0 <= index $2, '%') ? $2 : "%$2"; # Use the full GNU make style
-    my $to = (0 <= index $3, '%') ? $3 : "%$3";
+				# Substitution reference ('x:%.o=%.c' or 'x:o=c')?
+    my $pct = 0 <= index $2, '%'; # Is it the 1st form?
     $result = join ' ',
-      Mpp::Text::pattern_substitution $from, $to,
-	  split_on_whitespace expand_variable( $self, $1, $makefile_line );
+      Mpp::Text::pattern_substitution $pct ? ($2, $3) : ("%$2", "%$3"),
+	split_on_whitespace expand_variable( $self, $1, $makefile_line );
   } else {			# Must be a vanilla variable to expand.
     $result = expand_variable( $self, $expr, $makefile_line );
   }
@@ -321,7 +320,7 @@ sub expand_expression {
 }
 
 #
-# A variable allowing to intoduce special purpose Makefile variables that have
+# A variable allowing to introduce special purpose Makefile variables that have
 # a higher lookup priority than any other.  This is used by rules for target
 # specific variables and by $(foreach ...) for the index variable.
 # This must (typically temporarily) contain a hash reference with one or two
@@ -367,7 +366,7 @@ sub expand_variable {
 
 # 1st attempt:
     if( $var =~ /^\d+$/ || exists $Mpp::Subs::perl_unfriendly_symbols{$var} ) { # Is it one of the call
-				# or 1-char symbols like '$(11)' or '$@' that conflict with perl variables?
+				# or 1-char symbols like '$(11)' or '$@' that conflict with Perl variables?
 				# The call fn may eval more than got passed.  These can't be per target or global.
       if( ref $Mpp::Subs::perl_unfriendly_symbols{$var} ) {
 	$result = eval { &{$Mpp::Subs::perl_unfriendly_symbols{$var}}( undef, $self, $makefile_line ) };
@@ -427,6 +426,9 @@ sub expand_variable {
       my $tmp = !$Mpp::environment_override && $self->{ENVIRONMENT}{$var};
       if( $tmp && $fn == *{"Mpp::Subs::f_$var"}{CODE} ) {
 	$result = $tmp;
+      } elsif( defined $mode && $mode == 1 ) {
+	$result = "\$(perl &f_$var)"; # Turn it into a normal expression, so it can be extended.
+	$reexpand = 1;
       } else {
 	local $Mpp::makefile = $self; # Pass the function a reference to the makefile.
 	$result = &$fn( '', $self, $makefile_line ) and
@@ -537,7 +539,7 @@ sub implicitly_load {
 =head2 cleanup_vars
 
 Remove undefined variables from the environment and the command line.
-Such variables come into existence when perl looks for them, but they need
+Such variables come into existence when Perl looks for them, but they need
 to be ignored for comparing and/or setting them.
 
 =cut
@@ -909,7 +911,7 @@ sub load {
     # we might fail to re-rebuild it, and either way we don't need to.
     if($minfo->{RULE}) {
       require Mpp::BuildCheck::target_newer; # Make sure the method is loaded.
-      local $Mpp::default_build_check_method = $Mpp::BuildCheck::target_newer::target_newer;
+      local $Mpp::BuildCheck::default = $Mpp::BuildCheck::target_newer::target_newer;
 				# Use the target_newer technique for rebuilding
 				# makefiles, since makefiles are often modified
 				# by programs like configure which aren't
@@ -967,7 +969,7 @@ sub initialize {
     local $self->{INITIALIZED} = 1; # Don't go into deep recursion if expanding an export
     for my $var (keys %{$self->{EXPORTS}}) {
       $self->{EXPORTS}{$var} = expand_variable $self, $var, absolute_filename( $self->{MAKEFILE} ) . ':0';
-				# We don't know the line here, but provide one, in case perl code gets assigned.
+				# We don't know the line here, but provide one, in case Perl code gets assigned.
     }
 
     # NOTE: Don't set INITIALIZED here, because even though we've done the
@@ -1012,7 +1014,7 @@ sub assign {
     if( expand_variable $self, $name, $makefile_line, 2 ) {
       return $self;		# Nothing to do for ?= if already defined
     } else {
-      $type = 0;	# Else it's like a = assignment.
+      $type = 0;		# Else it's like a = assignment.
     }
   }
 
@@ -1331,7 +1333,7 @@ sub grok_rule {
 #
 # Pull off the : modifiers.
 #
-  my( $foreach, $signature, $build_check, $build_cache, $have_build_cache );
+  my( $foreach, $signature, $signature_name, $signature_override, $build_check, $build_cache, $have_build_cache );
   my( $lexer, $parser );
   my $conditional_scanning;
   my $multiple_rules_ok;
@@ -1365,9 +1367,11 @@ sub grok_rule {
     } elsif ($after_colon[-1] =~ /^\s*signature\s+(.*?)\s*$/) { # Specify signature class?
       $signature and die "$makefile_line: multiple :signature clauses\n";
       my $name = expand_text $self, $1, $makefile_line;
-      $signature = eval "use Mpp::Signature::$name; \$Mpp::Signature::${name}::$name" ||
-	eval "use Signature::$name; warn qq!$makefile_line: name Signature::$name is deprecated, rename to Mpp::Signature::$name\n!; \$Signature::${name}::$name";
-      unless( defined $signature ) {
+      $signature = Mpp::Signature::get( $name, $makefile_line );
+      if( defined $signature ) {
+	  $signature_name = $name;
+	  $signature_override = 1;
+      } else {
 # For backward compatibility, accept build check methods as a name to
 # :signature as well.
         $build_check = eval "use Mpp::BuildCheck::$name; \$Mpp::BuildCheck::${name}::$name" ||
@@ -1437,7 +1441,12 @@ sub grok_rule {
     pop @after_colon;
   }
   $build_check ||= $self->{DEFAULT_BUILD_CHECK_METHOD};
-  $signature ||= $self->{DEFAULT_SIGNATURE_METHOD};
+
+  ($signature, $signature_name, $signature_override) = $self->{DEFAULT_SIGNATURE_METHOD} ?
+    @$self{qw(DEFAULT_SIGNATURE_METHOD DEFAULT_SIG_METHOD_NAME DEFAULT_SIG_OVERRIDE)} :
+    ($Mpp::Signature::default, $Mpp::Signature::default_name, $Mpp::Signature::override)
+    unless $signature;
+
 #
 # Now process the pieces of the rule.  We recognize several different kinds
 # of rules:
@@ -1473,15 +1482,15 @@ sub grok_rule {
 # Convert GNU make's static pattern rules into something we like better.
 # If the rule was
 #    a.o b.o c.o : %.o : %.c
-# then we treat it as if it were written:
-#    $(foreach) : $(patsubst %.o, %.c, $(foreach)) : foreach a.o b.o c.o
+# then we treat it as if it were written (_ is internal arg for this purpose):
+#    $(foreach) : $(filesubst %.o, %.c, $(foreach),_) : foreach a.o b.o c.o
 #
   if (@after_colon == 2) {
     $foreach and die "$makefile_line: :foreach and GNU static pattern rule are incompatible\n";
     $foreach = $target_string;
     $after_colon[0] =~ /%/ or die "$makefile_line: no pattern in static pattern rule\n";
 				# Don't check for last chance, because we have finite set of targets
-    (@after_colon) = "\$(filesubst $after_colon[0], $after_colon[1], \$(foreach))";
+    (@after_colon) = "\$(filesubst $after_colon[0], $after_colon[1], \$(foreach),_)";
     $target_string = '$(foreach)';
   }
 
@@ -1550,12 +1559,12 @@ sub grok_rule {
 #
     if( $target_pattern ) { # Pattern rule?
 
-      $target_string = "\$(filesubst $deps[$pattern_dep], $target_string, \$(foreach))";
+      $target_string = "\$(filesubst $deps[$pattern_dep], $target_string, \$(foreach),_)";
       for( @deps[$pattern_dep+1..$#deps] ) { # Handle any extra dependencies:
 	index_ignoring_quotes( $_, '%' ) >= 0 and
-	  $_ = "\$(filesubst $deps[$pattern_dep], $_, \$(foreach))";
+	  $_ = "\$(filesubst $deps[$pattern_dep], $_, \$(foreach),_)";
       }
-      $include = "\$(filesubst $deps[$pattern_dep], $include, \$(foreach))"
+      $include = "\$(filesubst $deps[$pattern_dep], $include, \$(foreach),_)"
 	if $include && index_ignoring_quotes( $include, '%' ) >= 0;
       $deps[$pattern_dep] = '$(foreach)'; # This had better match the wildcard specified
 				# in the foreach clause.  I don't know of
@@ -1591,7 +1600,7 @@ sub grok_rule {
 				# Make the rule.
       local $Mpp::Subs::rule = $rule; # Put it so $(foreach) can properly expand.
       $build_check and $rule->set_build_check_method($build_check);
-      $signature and $rule->set_signature_method($signature);
+      $signature and $rule->set_signature_class( $signature_name, 0, $signature, $signature_override );
       $have_build_cache and $rule->set_build_cache($build_cache);
       $lexer and $rule->{LEXER} = $lexer;
       $parser and $rule->{PARSER} = $parser;
@@ -1655,7 +1664,7 @@ sub grok_rule {
 
       if (($is_double_colon ||	# Obsolete syntax?
 	   $action =~ /\$\@/) && # Does it include the old $@ target?
-	  $action !~ /\$([({])\1?(?:outputs|targets)\b/) {
+	  $action !~ /\$([({])\1?(?:outputs?|targets?)\b/) {
 				# And it doesn't include something that refers
 				# to all/many targets at once?
 	@target_exprs = @targets; # Apply rule independently to each target.
@@ -1668,7 +1677,7 @@ sub grok_rule {
         $rule->{DISPATCH} = $dispatch if $dispatch;
         $rule->{ENV_DEPENDENCY_STRING} = $env_dep_str if $env_dep_str;
 	$build_check and $rule->set_build_check_method($build_check);
-	$signature and $rule->set_signature_method($signature);
+	$signature and $rule->set_signature_class( $signature_name, 0, $signature, $signature_override );
         $have_build_cache and $rule->set_build_cache($build_cache);
 	$lexer and $rule->{LEXER} = $lexer;
 	$parser and $rule->{PARSER} = $parser;
@@ -1784,17 +1793,17 @@ sub grok_rule {
 #
 # Read a block either optionally indented {{ to }} or single braced.
 # The latter must finish on the same line or at the very beginning of
-# a following line.  Or upto the regexp given as 2nd arg
+# a following line.  Or up to the regexp given as 2nd arg
 #
 sub read_block {
-  my( $name, $code, $re ) = @_;		# Name the arguments.
+  my( $name, $code, $re, $strip ) = @_;		# Name the arguments.
   $re = $re ? qr/\s*$re\s*(?:)/ : ($code =~ /\{(\{?)/) && ($1 ? qr/\s*\}\}/ : qr/\}/);
 				# {{ is stronger than } at EOL
   if( $_[2] or $re ? $code !~ /\}\s*$/ : 1 ) { # Code is not entirely inline?
     $code .= "\n";		# Put the newline in that got removed.
     my $line;
     my $lineno = $makefile_lineno;
-    while (defined($line = defined wantarray ? &read_makefile_line : &_read_makefile_line_1 )) {
+    while (defined($line = $strip ? read_makefile_line_stripped( 1, 1 ) : defined wantarray ? &read_makefile_line : &_read_makefile_line_1 )) {
 				# Get the next line.
       $re ||= ($line =~ /^\s*\{\{/s) ? qr/\s*\}\}/ : ($line =~ /^\{/s) ? qr/\}/ : undef;
 				# Give {{ a chance on 2nd line.
@@ -1851,7 +1860,7 @@ sub read_makefile {
     $makefile_contents = <$fh>; # Read the whole makefile.
     $makefile_contents =~ tr/\r//d;
 				# Strip out those annoying CR characters
-				# which get put in sometimes on windows.
+				# which get put in sometimes on Windows.
   }
 
   if( $c_preprocess ) {
@@ -1936,7 +1945,7 @@ sub read_makefile {
 	  $equal = length;
 	  $_ .= '=';
 	}
-	$_ = read_block( define => $_, qr/endd?ef/ );
+	$_ = read_block( define => $_, qr/endd?ef/, 1 );
 	s/[ \t]*\\\n[ \t]*/ /g;
 	local $s_define = 1;
 	grok_assignment $self, $makefile_line, $equal, $keyword;
@@ -1946,11 +1955,9 @@ sub read_makefile {
 
     if( /^\s*([-\w]+)\s+(.*)/ ) { # Statement at beginning of line?
       my ($rtn, $rest_of_line) = ($1, $2);
-      $rtn =~ tr/-/_/;		# Make routine names more perl friendly.
+      $rtn =~ tr/-/_/;		# Make routine names more Perl friendly.
       substr $rtn, 0, 0, "$self->{PACKAGE}::s_";
       if( defined &$rtn ) {	# Function from makefile?
-	die "$makefile_line: export or override can not be combined with $rtn statement\n"
-	  if $keyword->{export} or $keyword->{override};
 	eval { &$rtn( $rest_of_line, $self, $makefile_line, $keyword ) };
 				# Try to call it as a subroutine.
 	die "$makefile_line: error handling $rtn statement\n$@\n" if $@;
@@ -2187,14 +2194,16 @@ sub _read_makefile_line_stripped_1 {
   }
 
   defined $line or return undef; # No point checking at end of file.
-  if( -1 < index $line, '$[' ) {
-    eval {
-      ++$expand_bracket;
-      unshift_makefile_lines( expand_text $makefile, $line, $makefile_name . ':' . $makefile_lineno );
+  if( -1 < (my $bracket = index $line, '$[') ) {
+    local $expand_bracket = 1;
+    $line = expand_text $makefile, $line, $makefile_name . ':' . $makefile_lineno;
+    if( -1 < index substr( $line, 0, -1 ), "\n" ) {
+      # TODO: figure out if we are in makeperl, else don't expand after #
+      warn "$makefile_name:$makefile_lineno: After # multiline \$[] only comments out 1st line.\n"
+	if -1 < index_ignoring_quotes substr( $line, 0, $bracket ), '#';
+      unshift_makefile_lines( $line );
       $line = shift @hold_lines;
-    };
-    --$expand_bracket;
-    die $@ if $@;
+    }
   }
   return $line if $_[1];
 
@@ -2274,7 +2283,7 @@ sub _read_makefile_line_stripped_1 {
 #
 # Skip until we find a line containing else or endif.  This is used to skip
 # over the false part of an if statement.  With a true argument always skip
-# upto endif.
+# up to endif.
 #
 sub skip_makefile_until_else_or_endif {
   my $was_true = $_[0];
@@ -2314,7 +2323,7 @@ sub skip_makefile_until_else_or_endif {
     } else {
       if ($line =~ /^\s*([-\w]+)/ ) {
 	my $rtn = $1;
-	$rtn =~ tr/-/_/;	# Make routine names more perl friendly.
+	$rtn =~ tr/-/_/;	# Make routine names more Perl friendly.
 	substr $rtn, 0, 0, 'Mpp::Subs::s_';
 	next if defined &$rtn;	# Statement?
       }
