@@ -1,4 +1,4 @@
-# $Id: Subs.pm,v 1.190 2011/09/15 21:08:07 pfeiffer Exp $
+# $Id: Subs.pm,v 1.198 2012/03/04 13:56:35 pfeiffer Exp $
 
 =head1 NAME
 
@@ -16,12 +16,12 @@ Subroutines in this package are called in two ways:
 Any line which isn't a rule or an assignment and has at the left margin a word
 is interpreted as a subroutine call to a subroutine in the makefile package,
 or if not in the makefile package, in this package.  "s_" is prefixed to the
-name before the perl function is looked up.
+name before the Perl function is looked up.
 
 =item 2)
 
 Any function that is in a make expression (e.g., $(xyz abc)) attempts to call
-a perl function in the make package, and failing that, in this package.  "f_"
+a Perl function in the make package, and failing that, in this package.  "f_"
 is prefixed to the name first.
 
 =back
@@ -49,7 +49,7 @@ use Mpp::CommandParser::Gcc;
 our( $makefile, $makefile_line );
 sub eval_or_die($$$) {
   my $code = $_[0];
-  # Make $makefile and $makefile_line available to the perl code, so that it
+  # Make $makefile and $makefile_line available to the Perl code, so that it
   # can call f_* and s_* subroutines.
   local( undef, $makefile, $makefile_line ) = @_; # Name the arguments.
 
@@ -266,7 +266,7 @@ sub relative_filenames {
 # a) The text after the function name in the makefile (with other macros
 #    already expanded).
 # b) The makefile.
-# c) The line number in the makefile that this expression occured in.
+# c) The line number in the makefile that this expression occurred in.
 #
 
 #
@@ -295,19 +295,19 @@ our %perl_unfriendly_symbols =
   );
 
 #
-# Obtain the single arg of a makefile function.
+# Obtain the single arg of a makefile f_function.
 # This utility takes the same 3 parameters as f_* functions, so call it as: &arg
 #
-# It gives you the expanded value of the callings function single arg, if the
+# It gives you the expanded value of the calling f_function's single arg, if the
 # first parameter is a ref to a string, else just the unexpanded string.
 # If the 2nd arg is false it also doesn't expand.
 #
-# If the function doesn't take an arg, there is no need to call this.
+# If the f_function doesn't take an arg, there is no need to call this.
 #
 sub arg { $_[1] && ref $_[0] ? $_[1]->expand_text( ${$_[0]}, $_[2] ) : $_[0] }
 
 #
-# Obtain multiple args of a makefile function.
+# Obtain multiple args of a makefile f_function.
 # This utility takes the same 3 parameters as arg
 #
 # Additional parameters:
@@ -316,7 +316,7 @@ sub arg { $_[1] && ref $_[0] ? $_[1]->expand_text( ${$_[0]}, $_[2] ) : $_[0] }
 # only_comma: don't eat space around commas
 #
 sub args {
-  local $_ = ref $_[0] ? ${$_[0]} : $_[0]; # Make a modifyable copy
+  local $_ = ref $_[0] ? ${$_[0]} : $_[0]; # Make a modifiable copy
   my $max = $_[3] || 2;
   my $min = ($_[4] or $max == ~0 ? 1 : $max) - 1;
   pos = 0;
@@ -394,6 +394,7 @@ sub f_call {
   my @args= args $_[0], $_[1], $_[2], ~0, 1, 1;
   local @perl_unfriendly_symbols{0..($#args>$call_args ? $#args : $call_args)} = @args; # assign to $0, $1, $2...
   local $call_args = $#args;
+  local $Mpp::Makefile::expand_bracket;
   $_[1]->expand_variable( $args[0], $_[2] );
 }
 
@@ -420,8 +421,9 @@ sub f_error {
 # will work with filesubst but not with patsubst.
 #
 sub f_filesubst {
-  my( $src, $dest, $words ) = args $_[0], $_[1], $_[2], 3;
+  my( $src, $dest, $words, $set_stem ) = args $_[0], $_[1], $_[2], 4, 3;
 				# Get the patterns.
+  die "$_[2]: filesubst has extra argument `$set_stem'\n" if defined $set_stem && $set_stem ne '_';
   my $cwd = $_[1]{CWD};
 #
 # First we eat away at the directories on the source until we find the
@@ -453,9 +455,11 @@ sub f_filesubst {
 				# directory.
   }
 
-  join ' ', Mpp::Text::pattern_substitution( case_sensitive_filenames ? $src : lc $src,
+  local $Mpp::Text::set_stem = 1 if $set_stem;
+  join ' ', Mpp::Text::pattern_substitution
+    case_sensitive_filenames ? $src : lc $src,
 					    $dest,
-					    @words );
+    @words;
 }
 
 sub f_filter {
@@ -715,7 +719,7 @@ sub f_iftrue {
 
 #
 # Infer the linker command from a list of objects.  If any of the objects
-# is fortran, we use $(FC) as a linker; if any of the objects is C++, we
+# is Fortran, we use $(FC) as a linker; if any of the objects is C++, we
 # use $(CXX); otherwise, we use $(CC).
 #
 # This function is mostly used by the default link rules (see
@@ -830,7 +834,7 @@ sub f_infer_objects {
       my $handle = when_done $bh, # Build this dependency.
       sub {			# Called when the build is finished:
 	defined($bh) && $bh->status and return $bh->status;
-				# Skip if an error occured.
+				# Skip if an error occurred.
 	my @this_sources = split /\01/, Mpp::File::build_info_string($o_info,'SORTED_DEPS') || '';
 				# Get the list of source files that went into
 				# it.
@@ -1168,7 +1172,7 @@ sub f_shell {
       warn "shell command `$str' returned `$?' at `$mkfile_line'\n";
   } else {
 #
-# We used to use perl's backquotes operators but these seem to have trouble,
+# We used to use Perl's backquotes operators but these seem to have trouble,
 # especially when doing parallel builds.  The backquote operator doesn't seem
 # to capture all of the output.	 Every once in a while (sometimes more often,
 # depending on system load and whether it's a parallel build) the backquote
@@ -1378,7 +1382,7 @@ sub f_sorted_dependencies {
 *f_sorted_inputs = *f_sorted_dependencies;
 
 #
-# Foreach is a little bit trick, since we have to support the new
+# Foreach is a little bit tricky, since we have to support the new
 # $(foreach) automatic variable, but also the old GNU make function
 # foreach.  We can tell the difference pretty easily by whether we have
 # any arguments.
@@ -1500,7 +1504,7 @@ sub s_build_cache {#_
       if $mkfile;		# Make sure we work even if cwd is wrong.
 
     require Mpp::BuildCache;	# Load the build cache mechanism.
-    warn $mkfile_line ? "$mkfile_line: " : '', "Setting another build cache.\n"
+    warn "$mkfile_line: Setting another build cache.\n"
       if $$var;
     $$var = new Mpp::BuildCache( $fname );
   }
@@ -1510,17 +1514,23 @@ sub s_build_cache {#_
 # Build_check statement.
 #
 sub s_build_check {#_
-  my( undef, $mkfile, $mkfile_line ) = @_;
-  my $name = $mkfile->expand_text( $_[0], $mkfile_line );
+  my( $name, $mkfile, $mkfile_line ) = @_;
+  my $global = delete $_[3]{global};
+  my $var = $global ? \$Mpp::BuildCheck::default : \$mkfile->{DEFAULT_BUILD_CHECK_METHOD};
+
+  $name = $mkfile->expand_text( $name, $mkfile_line )
+    if $mkfile;
   $name =~ s/^\s*(\w+)\s*$/$1/ or
     die "$mkfile_line: invalid build_check statement\n";
-  if( $name eq 'default' ) {	# Return to the default method?
-    delete $mkfile->{DEFAULT_BUILD_CHECK_METHOD};
-    return;
+  if( $name ne 'default' ) {
+    $$var = eval "use Mpp::BuildCheck::$name; \$Mpp::BuildCheck::${name}::$name" ||
+      eval "use BuildCheck::$name; warn qq!$mkfile_line: name BuildCheck::$name is deprecated, rename to Mpp::BuildCheck::$name\n!; \$BuildCheck::${name}::$name"
+      or die "$mkfile_line: invalid build_check method $name\n";
+  } elsif( $global ) {		# Return to the default method?
+    $$var = $Mpp::BuildCheck::exact_match::exact_match;
+  } else {
+    undef $$var;
   }
-  $mkfile->{DEFAULT_BUILD_CHECK_METHOD} = eval "use Mpp::BuildCheck::$name; \$Mpp::BuildCheck::${name}::$name" ||
-    eval "use BuildCheck::$name; warn qq!$mkfile_line: name BuildCheck::$name is deprecated, rename to Mpp::BuildCheck::$name\n!; \$BuildCheck::${name}::$name"
-    or die "$mkfile_line: invalid build_check method $name\n";
 }
 
 #
@@ -1694,14 +1704,14 @@ sub s_perl {#__
 				# Name the arguments.
   $perl_code = Mpp::Makefile::read_block( $keyword->{make} ? 'makeperl' : 'perl', $perl_code );
   $perl_code = $mkfile->expand_text($perl_code, $mkfile_line) if $keyword->{make};
-  $mkfile->cd;		# Make sure we're in the correct directory
-				# because some perl code will expect this.
+  $mkfile->cd;			# Make sure we're in the correct directory
+				# because some Perl code will expect this.
   eval_or_die $perl_code, $mkfile, $mkfile_line;
 }
 
 
 #
-# Begin a whole block of perl code.
+# Begin a whole block of Perl code.
 #
 sub s_perl_begin {#_
   my ($perl_code, $mkfile, $mkfile_line) = @_;
@@ -1710,7 +1720,7 @@ sub s_perl_begin {#_
     if $perl_code;
   $perl_code = Mpp::Makefile::read_block( perl_begin => $perl_code, qr/perl[-_]end/ );
   $mkfile->cd;			# Make sure we're in the correct directory
-				# because some perl code will expect this.
+				# because some Perl code will expect this.
   eval_or_die $perl_code, $mkfile, $mkfile_line;
 }
 
@@ -1732,14 +1742,12 @@ sub s_prebuild {#__
       die "failed to prebuild $target\n";
   }
 }
+*s_make = \&s_prebuild;
 sub prebuild {
   my ($finfo, $mkfile, $mkfile_line ) = @_;
-  my $myrule = Mpp::File::get_rule( $finfo );
   Mpp::log PREBUILD => $finfo, $mkfile_line
     if $Mpp::log_level;
-  if($myrule && !UNIVERSAL::isa($myrule, 'Mpp::DefaultRule') &&
-    !exists($finfo->{BUILD_HANDLE})
-  ) {
+  if( my $myrule = Mpp::File::get_rule $finfo ) {
     # If the file to be built is governed by the present Makefile, then
     # just initialize the Mpp::Makefile and build it based on what we know so far,
     # because then the file will *always* be built with the same limited
@@ -1750,10 +1758,11 @@ sub prebuild {
     # which Makefiles were loaded. Note that this warning isn't guaranteed to
     # show up when it's called for, because targets that are built via direct
     # calls to Mpp::build() don't undergo this check.
-    unless($myrule->makefile == $mkfile || $myrule->makefile->{INITIALIZED}) {
-      warn 'Attempting to build ' . absolute_filename( $finfo ) .
-	" before its makefile is completely loaded\n";
-    }
+    warn 'Attempting to build ' . &absolute_filename . " before its makefile is completely loaded\n"
+      unless ref( $myrule ) eq 'Mpp::DefaultRule' ||
+	exists $finfo->{BUILD_HANDLE} ||
+	$myrule->makefile == $mkfile ||
+	$myrule->makefile->{INITIALIZED};
   }
   Mpp::build($finfo);
 }
@@ -1861,32 +1870,46 @@ sub s_runtime {#__
 }
 
 #
-# Set the default signature method for all rules in this makefile:
+# Set the default signature method for all rules in this makefile or globally:
 #
 sub s_signature {#__
-  my( undef, $mkfile, $mkfile_line ) = @_;
-  my $name = $mkfile->expand_text( $_[0], $mkfile_line );
-  $name =~ s/^\s*(\w+)\s*$/$1/ or
-    die "$mkfile_line: invalid signature statement\n";
-  if( $name eq 'default' ) {	# Return to the default method?
-    delete $mkfile->{DEFAULT_SIGNATURE_METHOD}; # Get rid of any previous stored signature method.
-    return;
-  }
-  $mkfile->{DEFAULT_SIGNATURE_METHOD} = eval "use Mpp::Signature::$name; \$Mpp::Signature::${name}::$name" ||
-    eval "use Signature::$name; warn qq!$mkfile_line: name Signature::$name is deprecated, rename to Mpp::Signature::$name\n!; \$Signature::${name}::$name";
-  unless( defined $mkfile->{DEFAULT_SIGNATURE_METHOD} ) {
+  my( $name, $mkfile, $mkfile_line ) = @_;
+  my $global = delete $_[3]{global};
+  my $override = delete $_[3]{override};
+  my $var = $global ? \$Mpp::Signature::default : \$mkfile->{DEFAULT_SIGNATURE_METHOD};
+  my $name_var = $global ? \$Mpp::Signature::default_name : \$mkfile->{DEFAULT_SIG_METHOD_NAME};
+  my $override_var = $global ? \$Mpp::Signature::override : \$mkfile->{DEFAULT_SIG_OVERRIDE};
+  $name = $mkfile->expand_text( $name, $mkfile_line )
+    if $mkfile;
+  $name =~ s/^\s*(.*?)\s*$/$1/;
+  if( $name ne 'default' ) {
+    $$var = Mpp::Signature::get( $name, $mkfile_line );
+    if( defined $$var ) {
+      $$name_var = $name;
+      $$override_var = $override;
+    } else {
 #
 # The signature methods and build check methods used to be the same thing,
 # so for backward compatibility, see if this is actually a build check
 # method.
 #
-    $mkfile->{DEFAULT_BUILD_CHECK_METHOD} = eval "use Mpp::BuildCheck::$name; \$Mpp::BuildCheck::${name}::$name" ||
-      eval "use BuildCheck::$name; warn qq!$mkfile_line: name BuildCheck::$name is deprecated, rename to Mpp::BuildCheck::$name\n!; \$BuildCheck::${name}::$name";
-    if( defined $mkfile->{DEFAULT_BUILD_CHECK_METHOD} ) {
-      warn "$mkfile_line: requesting build check method $name via signature is deprecated.\n";
-    } else {
-      die "$mkfile_line: invalid signature method $name\n";
+      $var = eval "use Mpp::BuildCheck::$name; \$Mpp::BuildCheck::${name}::$name" ||
+	eval "use BuildCheck::$name; warn qq!$mkfile_line: name BuildCheck::$name is deprecated, rename to Mpp::BuildCheck::$name\n!; \$BuildCheck::${name}::$name";
+      if( defined $var ) {
+	warn "$mkfile_line: requesting build check method $name via signature is deprecated.\n";
+	if( $global ) {
+	  $Mpp::BuildCheck::default = $var;
+	} else {
+	  $mkfile->{DEFAULT_BUILD_CHECK_METHOD} = $var;
+	}
+      } else {
+	die "$mkfile_line: invalid signature method $name\n";
+      }
     }
+  } else {		# Return to the default method?
+    undef $$name_var;
+    undef $$var;
+    $$override_var = $override;
   }
 }
 
